# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import LlmResponse, LlmListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLlm:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Retell) -> None:
        llm = client.llm.create()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Retell) -> None:
        llm = client.llm.create(
            begin_message="Hey I am a virtual assistant calling from Retell Hospital.",
            default_dynamic_variables={"customer_name": "John Doe"},
            general_prompt="You are ...",
            general_tools=[
                {
                    "name": "end_call",
                    "type": "end_call",
                    "description": "End the call with user.",
                }
            ],
            knowledge_base_ids=["string"],
            model="gpt-4o",
            model_high_priority=True,
            model_temperature=0,
            s2s_model="gpt-4o-realtime",
            starting_state="information_collection",
            states=[
                {
                    "name": "information_collection",
                    "edges": [
                        {
                            "description": "Transition to book an appointment.",
                            "destination_state_name": "appointment_booking",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to collect information...",
                    "tools": [
                        {
                            "name": "transfer_to_support",
                            "transfer_destination": {
                                "number": "16175551212",
                                "type": "predefined",
                            },
                            "transfer_option": {
                                "type": "cold_transfer",
                                "show_transferee_as_caller": False,
                            },
                            "type": "transfer_call",
                            "description": "Transfer to the support team.",
                        }
                    ],
                },
                {
                    "name": "appointment_booking",
                    "edges": [
                        {
                            "description": "description",
                            "destination_state_name": "destination_state_name",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to book an appointment...",
                    "tools": [
                        {
                            "cal_api_key": "cal_live_xxxxxxxxxxxx",
                            "event_type_id": 60444,
                            "name": "book_appointment",
                            "type": "book_appointment_cal",
                            "description": "Book an annual check up.",
                            "timezone": "America/Los_Angeles",
                        }
                    ],
                },
            ],
            tool_call_strict_mode=True,
            version=0,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Retell) -> None:
        response = client.llm.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Retell) -> None:
        with client.llm.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        llm = client.llm.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Retell) -> None:
        llm = client.llm.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
            version=1,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.llm.with_raw_response.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.llm.with_streaming_response.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            client.llm.with_raw_response.retrieve(
                llm_id="",
            )

    @parametrize
    def test_method_update(self, client: Retell) -> None:
        llm = client.llm.update(
            llm_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Retell) -> None:
        llm = client.llm.update(
            llm_id="16b980523634a6dc504898cda492e939",
            query_version=1,
            begin_message="Hey I am a virtual assistant calling from Retell Hospital.",
            default_dynamic_variables={"customer_name": "John Doe"},
            general_prompt="You are ...",
            general_tools=[
                {
                    "name": "end_call",
                    "type": "end_call",
                    "description": "End the call with user.",
                }
            ],
            knowledge_base_ids=["string"],
            model="gpt-4o",
            model_high_priority=True,
            model_temperature=0,
            s2s_model="gpt-4o-realtime",
            starting_state="information_collection",
            states=[
                {
                    "name": "information_collection",
                    "edges": [
                        {
                            "description": "Transition to book an appointment.",
                            "destination_state_name": "appointment_booking",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to collect information...",
                    "tools": [
                        {
                            "name": "transfer_to_support",
                            "transfer_destination": {
                                "number": "16175551212",
                                "type": "predefined",
                            },
                            "transfer_option": {
                                "type": "cold_transfer",
                                "show_transferee_as_caller": False,
                            },
                            "type": "transfer_call",
                            "description": "Transfer to the support team.",
                        }
                    ],
                },
                {
                    "name": "appointment_booking",
                    "edges": [
                        {
                            "description": "description",
                            "destination_state_name": "destination_state_name",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to book an appointment...",
                    "tools": [
                        {
                            "cal_api_key": "cal_live_xxxxxxxxxxxx",
                            "event_type_id": 60444,
                            "name": "book_appointment",
                            "type": "book_appointment_cal",
                            "description": "Book an annual check up.",
                            "timezone": "America/Los_Angeles",
                        }
                    ],
                },
            ],
            tool_call_strict_mode=True,
            body_version=0,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Retell) -> None:
        response = client.llm.with_raw_response.update(
            llm_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Retell) -> None:
        with client.llm.with_streaming_response.update(
            llm_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            client.llm.with_raw_response.update(
                llm_id="",
            )

    @parametrize
    def test_method_list(self, client: Retell) -> None:
        llm = client.llm.list()
        assert_matches_type(LlmListResponse, llm, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.llm.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = response.parse()
        assert_matches_type(LlmListResponse, llm, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.llm.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = response.parse()
            assert_matches_type(LlmListResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Retell) -> None:
        llm = client.llm.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert llm is None

    @parametrize
    def test_raw_response_delete(self, client: Retell) -> None:
        response = client.llm.with_raw_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = response.parse()
        assert llm is None

    @parametrize
    def test_streaming_response_delete(self, client: Retell) -> None:
        with client.llm.with_streaming_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = response.parse()
            assert llm is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            client.llm.with_raw_response.delete(
                "",
            )


class TestAsyncLlm:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.create()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.create(
            begin_message="Hey I am a virtual assistant calling from Retell Hospital.",
            default_dynamic_variables={"customer_name": "John Doe"},
            general_prompt="You are ...",
            general_tools=[
                {
                    "name": "end_call",
                    "type": "end_call",
                    "description": "End the call with user.",
                }
            ],
            knowledge_base_ids=["string"],
            model="gpt-4o",
            model_high_priority=True,
            model_temperature=0,
            s2s_model="gpt-4o-realtime",
            starting_state="information_collection",
            states=[
                {
                    "name": "information_collection",
                    "edges": [
                        {
                            "description": "Transition to book an appointment.",
                            "destination_state_name": "appointment_booking",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to collect information...",
                    "tools": [
                        {
                            "name": "transfer_to_support",
                            "transfer_destination": {
                                "number": "16175551212",
                                "type": "predefined",
                            },
                            "transfer_option": {
                                "type": "cold_transfer",
                                "show_transferee_as_caller": False,
                            },
                            "type": "transfer_call",
                            "description": "Transfer to the support team.",
                        }
                    ],
                },
                {
                    "name": "appointment_booking",
                    "edges": [
                        {
                            "description": "description",
                            "destination_state_name": "destination_state_name",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to book an appointment...",
                    "tools": [
                        {
                            "cal_api_key": "cal_live_xxxxxxxxxxxx",
                            "event_type_id": 60444,
                            "name": "book_appointment",
                            "type": "book_appointment_cal",
                            "description": "Book an annual check up.",
                            "timezone": "America/Los_Angeles",
                        }
                    ],
                },
            ],
            tool_call_strict_mode=True,
            version=0,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRetell) -> None:
        response = await async_client.llm.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = await response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRetell) -> None:
        async with async_client.llm.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = await response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
            version=1,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.llm.with_raw_response.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = await response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.llm.with_streaming_response.retrieve(
            llm_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = await response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            await async_client.llm.with_raw_response.retrieve(
                llm_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.update(
            llm_id="16b980523634a6dc504898cda492e939",
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.update(
            llm_id="16b980523634a6dc504898cda492e939",
            query_version=1,
            begin_message="Hey I am a virtual assistant calling from Retell Hospital.",
            default_dynamic_variables={"customer_name": "John Doe"},
            general_prompt="You are ...",
            general_tools=[
                {
                    "name": "end_call",
                    "type": "end_call",
                    "description": "End the call with user.",
                }
            ],
            knowledge_base_ids=["string"],
            model="gpt-4o",
            model_high_priority=True,
            model_temperature=0,
            s2s_model="gpt-4o-realtime",
            starting_state="information_collection",
            states=[
                {
                    "name": "information_collection",
                    "edges": [
                        {
                            "description": "Transition to book an appointment.",
                            "destination_state_name": "appointment_booking",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to collect information...",
                    "tools": [
                        {
                            "name": "transfer_to_support",
                            "transfer_destination": {
                                "number": "16175551212",
                                "type": "predefined",
                            },
                            "transfer_option": {
                                "type": "cold_transfer",
                                "show_transferee_as_caller": False,
                            },
                            "type": "transfer_call",
                            "description": "Transfer to the support team.",
                        }
                    ],
                },
                {
                    "name": "appointment_booking",
                    "edges": [
                        {
                            "description": "description",
                            "destination_state_name": "destination_state_name",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                        }
                    ],
                    "state_prompt": "You will follow the steps below to book an appointment...",
                    "tools": [
                        {
                            "cal_api_key": "cal_live_xxxxxxxxxxxx",
                            "event_type_id": 60444,
                            "name": "book_appointment",
                            "type": "book_appointment_cal",
                            "description": "Book an annual check up.",
                            "timezone": "America/Los_Angeles",
                        }
                    ],
                },
            ],
            tool_call_strict_mode=True,
            body_version=0,
        )
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRetell) -> None:
        response = await async_client.llm.with_raw_response.update(
            llm_id="16b980523634a6dc504898cda492e939",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = await response.parse()
        assert_matches_type(LlmResponse, llm, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRetell) -> None:
        async with async_client.llm.with_streaming_response.update(
            llm_id="16b980523634a6dc504898cda492e939",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = await response.parse()
            assert_matches_type(LlmResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            await async_client.llm.with_raw_response.update(
                llm_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.list()
        assert_matches_type(LlmListResponse, llm, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.llm.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = await response.parse()
        assert_matches_type(LlmListResponse, llm, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.llm.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = await response.parse()
            assert_matches_type(LlmListResponse, llm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncRetell) -> None:
        llm = await async_client.llm.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )
        assert llm is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncRetell) -> None:
        response = await async_client.llm.with_raw_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        llm = await response.parse()
        assert llm is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncRetell) -> None:
        async with async_client.llm.with_streaming_response.delete(
            "oBeDLoLOeuAbiuaMFXRtDOLriTJ5tSxD",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            llm = await response.parse()
            assert llm is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `llm_id` but received ''"):
            await async_client.llm.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, Iterable, cast

import httpx

from ..types import knowledge_base_create_params, knowledge_base_add_sources_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.knowledge_base_response import KnowledgeBaseResponse
from ..types.knowledge_base_list_response import KnowledgeBaseListResponse

__all__ = ["KnowledgeBaseResource", "AsyncKnowledgeBaseResource"]


class KnowledgeBaseResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KnowledgeBaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return KnowledgeBaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeBaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return KnowledgeBaseResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        knowledge_base_name: str,
        enable_auto_refresh: bool | NotGiven = NOT_GIVEN,
        knowledge_base_files: List[FileTypes] | NotGiven = NOT_GIVEN,
        knowledge_base_texts: Iterable[knowledge_base_create_params.KnowledgeBaseText] | NotGiven = NOT_GIVEN,
        knowledge_base_urls: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Create a new knowledge base

        Args:
          knowledge_base_name: Name of the knowledge base. Must be less than 40 characters.

          enable_auto_refresh: Whether to enable auto refresh for the knowledge base urls. If set to true, will
              retrieve the data from the specified url every 12 hours.

          knowledge_base_files: Files to add to the knowledge base. Limit to 25 files, where each file is
              limited to 50MB.

          knowledge_base_texts: Texts to add to the knowledge base.

          knowledge_base_urls: URLs to be scraped and added to the knowledge base. Must be valid urls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "knowledge_base_name": knowledge_base_name,
                "enable_auto_refresh": enable_auto_refresh,
                "knowledge_base_files": knowledge_base_files,
                "knowledge_base_texts": knowledge_base_texts,
                "knowledge_base_urls": knowledge_base_urls,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["knowledge_base_files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/create-knowledge-base",
            body=maybe_transform(body, knowledge_base_create_params.KnowledgeBaseCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    def retrieve(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Retrieve details of a specific knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get(
            f"/get-knowledge-base/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseListResponse:
        """List all knowledge bases"""
        return self._get(
            "/list-knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseListResponse,
        )

    def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an existing knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/delete-knowledge-base/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def add_sources(
        self,
        knowledge_base_id: str,
        *,
        knowledge_base_files: List[FileTypes] | NotGiven = NOT_GIVEN,
        knowledge_base_texts: Iterable[knowledge_base_add_sources_params.KnowledgeBaseText] | NotGiven = NOT_GIVEN,
        knowledge_base_urls: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Add sources to a knowledge base

        Args:
          knowledge_base_files: Files to add to the knowledge base. Limit to 25 files, where each file is
              limited to 50MB.

          knowledge_base_texts: Texts to add to the knowledge base.

          knowledge_base_urls: URLs to be scraped and added to the knowledge base. Must be valid urls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        body = deepcopy_minimal(
            {
                "knowledge_base_files": knowledge_base_files,
                "knowledge_base_texts": knowledge_base_texts,
                "knowledge_base_urls": knowledge_base_urls,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["knowledge_base_files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/add-knowledge-base-sources/{knowledge_base_id}",
            body=maybe_transform(body, knowledge_base_add_sources_params.KnowledgeBaseAddSourcesParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    def delete_source(
        self,
        source_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Delete an existing source from knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not source_id:
            raise ValueError(f"Expected a non-empty value for `source_id` but received {source_id!r}")
        return self._delete(
            f"/delete-knowledge-base-source/{knowledge_base_id}/source/{source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )


class AsyncKnowledgeBaseResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeBaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeBaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeBaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncKnowledgeBaseResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        knowledge_base_name: str,
        enable_auto_refresh: bool | NotGiven = NOT_GIVEN,
        knowledge_base_files: List[FileTypes] | NotGiven = NOT_GIVEN,
        knowledge_base_texts: Iterable[knowledge_base_create_params.KnowledgeBaseText] | NotGiven = NOT_GIVEN,
        knowledge_base_urls: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Create a new knowledge base

        Args:
          knowledge_base_name: Name of the knowledge base. Must be less than 40 characters.

          enable_auto_refresh: Whether to enable auto refresh for the knowledge base urls. If set to true, will
              retrieve the data from the specified url every 12 hours.

          knowledge_base_files: Files to add to the knowledge base. Limit to 25 files, where each file is
              limited to 50MB.

          knowledge_base_texts: Texts to add to the knowledge base.

          knowledge_base_urls: URLs to be scraped and added to the knowledge base. Must be valid urls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "knowledge_base_name": knowledge_base_name,
                "enable_auto_refresh": enable_auto_refresh,
                "knowledge_base_files": knowledge_base_files,
                "knowledge_base_texts": knowledge_base_texts,
                "knowledge_base_urls": knowledge_base_urls,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["knowledge_base_files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/create-knowledge-base",
            body=await async_maybe_transform(body, knowledge_base_create_params.KnowledgeBaseCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    async def retrieve(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Retrieve details of a specific knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._get(
            f"/get-knowledge-base/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseListResponse:
        """List all knowledge bases"""
        return await self._get(
            "/list-knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseListResponse,
        )

    async def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an existing knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/delete-knowledge-base/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def add_sources(
        self,
        knowledge_base_id: str,
        *,
        knowledge_base_files: List[FileTypes] | NotGiven = NOT_GIVEN,
        knowledge_base_texts: Iterable[knowledge_base_add_sources_params.KnowledgeBaseText] | NotGiven = NOT_GIVEN,
        knowledge_base_urls: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Add sources to a knowledge base

        Args:
          knowledge_base_files: Files to add to the knowledge base. Limit to 25 files, where each file is
              limited to 50MB.

          knowledge_base_texts: Texts to add to the knowledge base.

          knowledge_base_urls: URLs to be scraped and added to the knowledge base. Must be valid urls.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        body = deepcopy_minimal(
            {
                "knowledge_base_files": knowledge_base_files,
                "knowledge_base_texts": knowledge_base_texts,
                "knowledge_base_urls": knowledge_base_urls,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["knowledge_base_files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/add-knowledge-base-sources/{knowledge_base_id}",
            body=await async_maybe_transform(body, knowledge_base_add_sources_params.KnowledgeBaseAddSourcesParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )

    async def delete_source(
        self,
        source_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseResponse:
        """
        Delete an existing source from knowledge base

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not source_id:
            raise ValueError(f"Expected a non-empty value for `source_id` but received {source_id!r}")
        return await self._delete(
            f"/delete-knowledge-base-source/{knowledge_base_id}/source/{source_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeBaseResponse,
        )


class KnowledgeBaseResourceWithRawResponse:
    def __init__(self, knowledge_base: KnowledgeBaseResource) -> None:
        self._knowledge_base = knowledge_base

        self.create = to_raw_response_wrapper(
            knowledge_base.create,
        )
        self.retrieve = to_raw_response_wrapper(
            knowledge_base.retrieve,
        )
        self.list = to_raw_response_wrapper(
            knowledge_base.list,
        )
        self.delete = to_raw_response_wrapper(
            knowledge_base.delete,
        )
        self.add_sources = to_raw_response_wrapper(
            knowledge_base.add_sources,
        )
        self.delete_source = to_raw_response_wrapper(
            knowledge_base.delete_source,
        )


class AsyncKnowledgeBaseResourceWithRawResponse:
    def __init__(self, knowledge_base: AsyncKnowledgeBaseResource) -> None:
        self._knowledge_base = knowledge_base

        self.create = async_to_raw_response_wrapper(
            knowledge_base.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            knowledge_base.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            knowledge_base.list,
        )
        self.delete = async_to_raw_response_wrapper(
            knowledge_base.delete,
        )
        self.add_sources = async_to_raw_response_wrapper(
            knowledge_base.add_sources,
        )
        self.delete_source = async_to_raw_response_wrapper(
            knowledge_base.delete_source,
        )


class KnowledgeBaseResourceWithStreamingResponse:
    def __init__(self, knowledge_base: KnowledgeBaseResource) -> None:
        self._knowledge_base = knowledge_base

        self.create = to_streamed_response_wrapper(
            knowledge_base.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            knowledge_base.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            knowledge_base.list,
        )
        self.delete = to_streamed_response_wrapper(
            knowledge_base.delete,
        )
        self.add_sources = to_streamed_response_wrapper(
            knowledge_base.add_sources,
        )
        self.delete_source = to_streamed_response_wrapper(
            knowledge_base.delete_source,
        )


class AsyncKnowledgeBaseResourceWithStreamingResponse:
    def __init__(self, knowledge_base: AsyncKnowledgeBaseResource) -> None:
        self._knowledge_base = knowledge_base

        self.create = async_to_streamed_response_wrapper(
            knowledge_base.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            knowledge_base.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            knowledge_base.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            knowledge_base.delete,
        )
        self.add_sources = async_to_streamed_response_wrapper(
            knowledge_base.add_sources,
        )
        self.delete_source = async_to_streamed_response_wrapper(
            knowledge_base.delete_source,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import batch_call_create_batch_call_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.batch_call_response import BatchCallResponse

__all__ = ["BatchCallResource", "AsyncBatchCallResource"]


class BatchCallResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BatchCallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return BatchCallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchCallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return BatchCallResourceWithStreamingResponse(self)

    def create_batch_call(
        self,
        *,
        from_number: str,
        tasks: Iterable[batch_call_create_batch_call_params.Task],
        name: str | NotGiven = NOT_GIVEN,
        trigger_timestamp: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchCallResponse:
        """Create a batch call

        Args:
          from_number: The number you own in E.164 format.

        Must be a number purchased from Retell or
              imported to Retell.

          tasks: A list of individual call tasks to be executed as part of the batch call. Each
              task represents a single outbound call and includes details such as the
              recipient's phone number and optional dynamic variables to personalize the call
              content.

          name: The name of the batch call. Only used for your own reference.

          trigger_timestamp: The scheduled time for sending the batch call, represented as a Unix timestamp
              in milliseconds. If omitted, the call will be sent immediately.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-batch-call",
            body=maybe_transform(
                {
                    "from_number": from_number,
                    "tasks": tasks,
                    "name": name,
                    "trigger_timestamp": trigger_timestamp,
                },
                batch_call_create_batch_call_params.BatchCallCreateBatchCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCallResponse,
        )


class AsyncBatchCallResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBatchCallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchCallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchCallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncBatchCallResourceWithStreamingResponse(self)

    async def create_batch_call(
        self,
        *,
        from_number: str,
        tasks: Iterable[batch_call_create_batch_call_params.Task],
        name: str | NotGiven = NOT_GIVEN,
        trigger_timestamp: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchCallResponse:
        """Create a batch call

        Args:
          from_number: The number you own in E.164 format.

        Must be a number purchased from Retell or
              imported to Retell.

          tasks: A list of individual call tasks to be executed as part of the batch call. Each
              task represents a single outbound call and includes details such as the
              recipient's phone number and optional dynamic variables to personalize the call
              content.

          name: The name of the batch call. Only used for your own reference.

          trigger_timestamp: The scheduled time for sending the batch call, represented as a Unix timestamp
              in milliseconds. If omitted, the call will be sent immediately.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-batch-call",
            body=await async_maybe_transform(
                {
                    "from_number": from_number,
                    "tasks": tasks,
                    "name": name,
                    "trigger_timestamp": trigger_timestamp,
                },
                batch_call_create_batch_call_params.BatchCallCreateBatchCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BatchCallResponse,
        )


class BatchCallResourceWithRawResponse:
    def __init__(self, batch_call: BatchCallResource) -> None:
        self._batch_call = batch_call

        self.create_batch_call = to_raw_response_wrapper(
            batch_call.create_batch_call,
        )


class AsyncBatchCallResourceWithRawResponse:
    def __init__(self, batch_call: AsyncBatchCallResource) -> None:
        self._batch_call = batch_call

        self.create_batch_call = async_to_raw_response_wrapper(
            batch_call.create_batch_call,
        )


class BatchCallResourceWithStreamingResponse:
    def __init__(self, batch_call: BatchCallResource) -> None:
        self._batch_call = batch_call

        self.create_batch_call = to_streamed_response_wrapper(
            batch_call.create_batch_call,
        )


class AsyncBatchCallResourceWithStreamingResponse:
    def __init__(self, batch_call: AsyncBatchCallResource) -> None:
        self._batch_call = batch_call

        self.create_batch_call = async_to_streamed_response_wrapper(
            batch_call.create_batch_call,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    KnowledgeBaseResponse,
    KnowledgeBaseListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledgeBase:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_method_create(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.create(
            knowledge_base_name="Sample KB",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_method_create_with_all_params(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.create(
            knowledge_base_name="Sample KB",
            enable_auto_refresh=True,
            knowledge_base_files=[b"raw file contents"],
            knowledge_base_texts=[
                {
                    "text": "text",
                    "title": "title",
                }
            ],
            knowledge_base_urls=["https://www.example.com", "https://www.retellai.com"],
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_raw_response_create(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.create(
            knowledge_base_name="Sample KB",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_streaming_response_create(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.create(
            knowledge_base_name="Sample KB",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.retrieve(
            "kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.retrieve(
            "kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.retrieve(
            "kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_base.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.list()
        assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.delete(
            "kb_1234567890",
        )
        assert knowledge_base is None

    @parametrize
    def test_raw_response_delete(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.delete(
            "kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert knowledge_base is None

    @parametrize
    def test_streaming_response_delete(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.delete(
            "kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert knowledge_base is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_base.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_method_add_sources(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.add_sources(
            knowledge_base_id="kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_method_add_sources_with_all_params(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.add_sources(
            knowledge_base_id="kb_1234567890",
            knowledge_base_files=[b"raw file contents"],
            knowledge_base_texts=[
                {
                    "text": "text",
                    "title": "title",
                }
            ],
            knowledge_base_urls=["https://www.example.com", "https://www.retellai.com"],
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_raw_response_add_sources(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.add_sources(
            knowledge_base_id="kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_streaming_response_add_sources(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.add_sources(
            knowledge_base_id="kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="custom code")
    @parametrize
    def test_path_params_add_sources(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_base.with_raw_response.add_sources(
                knowledge_base_id="",
            )

    @parametrize
    def test_method_delete_source(self, client: Retell) -> None:
        knowledge_base = client.knowledge_base.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_raw_response_delete_source(self, client: Retell) -> None:
        response = client.knowledge_base.with_raw_response.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    def test_streaming_response_delete_source(self, client: Retell) -> None:
        with client.knowledge_base.with_streaming_response.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete_source(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            client.knowledge_base.with_raw_response.delete_source(
                source_id="source_1234567890",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `source_id` but received ''"):
            client.knowledge_base.with_raw_response.delete_source(
                source_id="",
                knowledge_base_id="kb_1234567890",
            )


class TestAsyncKnowledgeBase:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_method_create(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.create(
            knowledge_base_name="Sample KB",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.create(
            knowledge_base_name="Sample KB",
            enable_auto_refresh=True,
            knowledge_base_files=[b"raw file contents"],
            knowledge_base_texts=[
                {
                    "text": "text",
                    "title": "title",
                }
            ],
            knowledge_base_urls=["https://www.example.com", "https://www.retellai.com"],
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.create(
            knowledge_base_name="Sample KB",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.create(
            knowledge_base_name="Sample KB",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.retrieve(
            "kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.retrieve(
            "kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.retrieve(
            "kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_base.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.list()
        assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseListResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.delete(
            "kb_1234567890",
        )
        assert knowledge_base is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.delete(
            "kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert knowledge_base is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.delete(
            "kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert knowledge_base is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_base.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_method_add_sources(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.add_sources(
            knowledge_base_id="kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_method_add_sources_with_all_params(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.add_sources(
            knowledge_base_id="kb_1234567890",
            knowledge_base_files=[b"raw file contents"],
            knowledge_base_texts=[
                {
                    "text": "text",
                    "title": "title",
                }
            ],
            knowledge_base_urls=["https://www.example.com", "https://www.retellai.com"],
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_raw_response_add_sources(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.add_sources(
            knowledge_base_id="kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_streaming_response_add_sources(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.add_sources(
            knowledge_base_id="kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="custom code")
    @parametrize
    async def test_path_params_add_sources(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_base.with_raw_response.add_sources(
                knowledge_base_id="",
            )

    @parametrize
    async def test_method_delete_source(self, async_client: AsyncRetell) -> None:
        knowledge_base = await async_client.knowledge_base.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        )
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_raw_response_delete_source(self, async_client: AsyncRetell) -> None:
        response = await async_client.knowledge_base.with_raw_response.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

    @parametrize
    async def test_streaming_response_delete_source(self, async_client: AsyncRetell) -> None:
        async with async_client.knowledge_base.with_streaming_response.delete_source(
            source_id="source_1234567890",
            knowledge_base_id="kb_1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete_source(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `knowledge_base_id` but received ''"):
            await async_client.knowledge_base.with_raw_response.delete_source(
                source_id="source_1234567890",
                knowledge_base_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `source_id` but received ''"):
            await async_client.knowledge_base.with_raw_response.delete_source(
                source_id="",
                knowledge_base_id="kb_1234567890",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["BatchCallCreateBatchCallParams", "Task"]


class BatchCallCreateBatchCallParams(TypedDict, total=False):
    from_number: Required[str]
    """The number you own in E.164 format.

    Must be a number purchased from Retell or imported to Retell.
    """

    tasks: Required[Iterable[Task]]
    """A list of individual call tasks to be executed as part of the batch call.

    Each task represents a single outbound call and includes details such as the
    recipient's phone number and optional dynamic variables to personalize the call
    content.
    """

    name: str
    """The name of the batch call. Only used for your own reference."""

    trigger_timestamp: float
    """
    The scheduled time for sending the batch call, represented as a Unix timestamp
    in milliseconds. If omitted, the call will be sent immediately.
    """


class Task(TypedDict, total=False):
    to_number: Required[str]
    """The The number you want to call, in E.164 format.

    If using a number purchased from Retell, only US numbers are supported as
    destination.
    """

    retell_llm_dynamic_variables: Dict[str, object]
    """
    Add optional dynamic variables in key value pairs of string that injects into
    your Response Engine prompt and tool description. Only applicable for Response
    Engine.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..types import chat_create_params, chat_create_chat_completion_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.chat_response import ChatResponse
from ..types.chat_list_response import ChatListResponse
from ..types.chat_create_chat_completion_response import ChatCreateChatCompletionResponse

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        agent_id: str,
        agent_version: int | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        retell_llm_dynamic_variables: Dict[str, object] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatResponse:
        """
        Create a chat session

        Args:
          agent_id: The chat agent to use for the call.

          agent_version: The version of the chat agent to use for the call.

          metadata: An arbitrary object for storage purpose only. You can put anything here like
              your internal customer id associated with the chat. Not used for processing. You
              can later get this field from the chat object.

          retell_llm_dynamic_variables: Add optional dynamic variables in key value pairs of string that injects into
              your Response Engine prompt and tool description. Only applicable for Response
              Engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-chat",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_version": agent_version,
                    "metadata": metadata,
                    "retell_llm_dynamic_variables": retell_llm_dynamic_variables,
                },
                chat_create_params.ChatCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatResponse,
        )

    def retrieve(
        self,
        chat_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatResponse:
        """
        Retrieve details of a specific chat

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        return self._get(
            f"/get-chat/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatListResponse:
        """List all chats"""
        return self._get(
            "/list-chat",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatListResponse,
        )

    def create_chat_completion(
        self,
        *,
        chat_id: str,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCreateChatCompletionResponse:
        """
        Create a chat completion message

        Args:
          chat_id: Unique id of the chat to create completion.

          content: user message to generate agent chat completion.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/create-chat-completion",
            body=maybe_transform(
                {
                    "chat_id": chat_id,
                    "content": content,
                },
                chat_create_chat_completion_params.ChatCreateChatCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateChatCompletionResponse,
        )

    def end(
        self,
        chat_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        End an ongoing chat

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/end-chat/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        agent_id: str,
        agent_version: int | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        retell_llm_dynamic_variables: Dict[str, object] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatResponse:
        """
        Create a chat session

        Args:
          agent_id: The chat agent to use for the call.

          agent_version: The version of the chat agent to use for the call.

          metadata: An arbitrary object for storage purpose only. You can put anything here like
              your internal customer id associated with the chat. Not used for processing. You
              can later get this field from the chat object.

          retell_llm_dynamic_variables: Add optional dynamic variables in key value pairs of string that injects into
              your Response Engine prompt and tool description. Only applicable for Response
              Engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-chat",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_version": agent_version,
                    "metadata": metadata,
                    "retell_llm_dynamic_variables": retell_llm_dynamic_variables,
                },
                chat_create_params.ChatCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatResponse,
        )

    async def retrieve(
        self,
        chat_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatResponse:
        """
        Retrieve details of a specific chat

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        return await self._get(
            f"/get-chat/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatListResponse:
        """List all chats"""
        return await self._get(
            "/list-chat",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatListResponse,
        )

    async def create_chat_completion(
        self,
        *,
        chat_id: str,
        content: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCreateChatCompletionResponse:
        """
        Create a chat completion message

        Args:
          chat_id: Unique id of the chat to create completion.

          content: user message to generate agent chat completion.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/create-chat-completion",
            body=await async_maybe_transform(
                {
                    "chat_id": chat_id,
                    "content": content,
                },
                chat_create_chat_completion_params.ChatCreateChatCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateChatCompletionResponse,
        )

    async def end(
        self,
        chat_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        End an ongoing chat

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not chat_id:
            raise ValueError(f"Expected a non-empty value for `chat_id` but received {chat_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/end-chat/{chat_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create = to_raw_response_wrapper(
            chat.create,
        )
        self.retrieve = to_raw_response_wrapper(
            chat.retrieve,
        )
        self.list = to_raw_response_wrapper(
            chat.list,
        )
        self.create_chat_completion = to_raw_response_wrapper(
            chat.create_chat_completion,
        )
        self.end = to_raw_response_wrapper(
            chat.end,
        )


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create = async_to_raw_response_wrapper(
            chat.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            chat.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            chat.list,
        )
        self.create_chat_completion = async_to_raw_response_wrapper(
            chat.create_chat_completion,
        )
        self.end = async_to_raw_response_wrapper(
            chat.end,
        )


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create = to_streamed_response_wrapper(
            chat.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            chat.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            chat.list,
        )
        self.create_chat_completion = to_streamed_response_wrapper(
            chat.create_chat_completion,
        )
        self.end = to_streamed_response_wrapper(
            chat.end,
        )


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create = async_to_streamed_response_wrapper(
            chat.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            chat.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            chat.list,
        )
        self.create_chat_completion = async_to_streamed_response_wrapper(
            chat.create_chat_completion,
        )
        self.end = async_to_streamed_response_wrapper(
            chat.end,
        )
